# $Id: oaiserver.py 3953 2010-09-13 15:12:30Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.7.0/src/oaiserver.py $
#!/usr/bin/env python2.5

from meresco.core import Transparant, Observable, be

from meresco.components import readConfig, XmlPrintLxml, RewritePartname, XmlParseLxml
#from meresco.components import readConfig, StorageComponent, XmlPrintLxml, RewritePartname, XmlParseLxml
from meresco.components.http import ObservableHttpServer, PathFilter, IpFilter

from meresco.oai import OaiJazz, OaiPmh, OaiAddRecordWithDefaults
from meresco.components.facetindex import LuceneIndex
from meresco.components.facetindex.tools import unlock
from meresco.components.http.webrequestserver import WebRequestServer

from weightless import Reactor, compose
from cq2utils import DirectoryWatcher

from os.path import dirname, join, isdir
from os import rename, makedirs
from time import strftime, localtime
from sys import stdout, argv, stderr

from glob import glob

from bzv.utils import HandleRequestPrinter, IdentifierPrinter, Printer, MessageLogger
from bzv import EditorialItemProcessor, Mods2Dc
from bzv.elsevierreader import readToc
from bzv.storagecomponent import BZVStorageComponent

class PartnameFilter(Transparant):
    def __init__(self, partnames):
        Transparant.__init__(self)
        self._partnames = partnames

    def add(self, id, partname, data):
        if partname in self._partnames:
            yield self.all.add(id, partname, data)

class swallowStopIteration:

    def __init__(self, f):
        self._f = f

    def __call__(self):
        try:
            self._f()
        except StopIteration:
            pass

class ProcessElsevierData(object):

    def __init__(self, reactor, event, inboxDirectory, doneDirectory, editorialItemProcessor):
        self._reactor = reactor
        self._donePath = join(doneDirectory, event.name)
        self._tocDirname = join(inboxDirectory, event.name)
        self._tocFile = open(join(inboxDirectory, event.name, 'dataset.toc'))

        self._editorialItemProcessor = editorialItemProcessor

        generator = self.do()
        self._reactor.addReader(self._tocFile, swallowStopIteration(generator.next))

    def do(self):
        #generatorExit?
        try:
            f = self._editorialItemProcessor.process(self._tocDirname)
            f.next()
            generator = compose(readToc(f))
            generator.next()

            yield
            line = self._tocFile.readline()
            while line:
                generator.send(line)
                yield
                line = self._tocFile.readline()
            generator.close()

            rename(self._tocDirname, self._donePath + strftime('_%Y%m%d-%H%M%S', localtime()))

        finally:
            self._reactor.removeReader(self._tocFile)



def main(reactor, observableHttpServer, inboxDirectory, doneDirectory, databasePath, oaiAdminEmail, oaiRepositoryName, stdoutStream, stderrStream):

    oaiStorageDirectory = join(databasePath, 'storage')
    oaiDataDirectory = join(databasePath, 'data')

    oaiStorageComponent = BZVStorageComponent(oaiStorageDirectory)

    editorialItemProcessor = EditorialItemProcessor()

    oaiJazzComponent = OaiJazz(oaiDataDirectory)

    if glob(inboxDirectory + "/*"):
        raise Exception('inboxDirectory %s not empty: %s' % (inboxDirectory, glob(inboxDirectory + "/*")))

    isdir(inboxDirectory) or makedirs(inboxDirectory)
    isdir(doneDirectory) or makedirs(doneDirectory)

    directoryWatcher = DirectoryWatcher(
        inboxDirectory,
        lambda event: ProcessElsevierData(reactor, event, inboxDirectory, doneDirectory, editorialItemProcessor),
        CreateFile=True, ModifyFile=True, MoveInFile=True)
    reactor.addReader(directoryWatcher, directoryWatcher)

    processor = \
    (Observable(),
        (editorialItemProcessor,
            (oaiStorageComponent,),
            (PartnameFilter(['mods']),
                (XmlParseLxml(),
                    (Mods2Dc(),
                        (RewritePartname('oai_dc'),
                            (XmlPrintLxml(),
                                (oaiStorageComponent,),
                            )
                        )
                    ),
                    (OaiAddRecordWithDefaults(
                        metadataFormats=[
                            ('mods', 'http://www.loc.gov/standards/mods/v3/mods-3-3.xsd', 'http://www.loc.gov/mods/v3'),
                            ('oai_dc', 'http://www.openarchives.org/OAI/2.0/oai_dc.xsd', 'http://www.openarchives.org/OAI/2.0/oai_dc/')
                        ],
                        sets=[('elsevier','elsevier')]),
                        (oaiJazzComponent,),
                    )
                ),
            ),
            (MessageLogger(stdoutStream,stderrStream),)
        )
    )

    server = \
    (Observable(),
        (observableHttpServer,
            (IpFilter(allowedIps=[
                '131.180.162.26',       # Discover development
                '131.180.162.28',       # Discover productie
                '131.180.162.27',       # Discover testing
                '127.0.0.1',
                      ],
                      allowedIpRanges=[
                 ('137.56.0.0', '137.56.255.255'), # Tilburg
                 ('169.254.0.0', '169.254.255.255') # internal range acceptatie
                      ]
                     ),
               (PathFilter('/elsevier/oai'),
                   (HandleRequestPrinter(),),
                   (WebRequestServer(),
                       (OaiPmh(repositoryName=oaiRepositoryName,
                           adminEmail=oaiAdminEmail),
                           (oaiJazzComponent,),
                           (oaiStorageComponent,),
                       )
                   )
               )
            )
        )
    )
    return processor, server
